//###################################################################### Mzf.js
// Purpose...: Record/play/send Sharp MZ-700 MZF files
// Input.....: $1: Function to perform
//                 -R: Call CoolEdit to record from SAVE, then convert the
//                     WAV file to MZF
//                 -P: Convert MZF file to to WAV and play using WinAmp
//                 -S: Convert MZF file to binary and send to RCVBIN
//             $2: Input file with drive/path
// Version...: 2025.04.16
//#############################################################################

//**************************************** declare global objects and variables
var m_acVersion   = "25.04.16"
var m_oArgs       = WScript.Arguments;
var m_oShell      = WScript.CreateObject("WScript.Shell");
var m_oFSys       = WScript.CreateObject("Scripting.FileSystemObject");
var m_acTempDir   = m_oShell.ExpandEnvironmentStrings("%TEMP%" + "\\");
var m_acPrograms  = m_oShell.ExpandEnvironmentStrings("%AxPrograms%");
var m_acWorkspace = m_oShell.ExpandEnvironmentStrings("%AxWorkspace%");
var m_acMzsHome   = "U:\\Sharp\\MZ-700\\Tools\\MzfStudio\\";
var m_acCoolIni   = "U:\\Sharp\\MZ-700\\Tools\\CoolEdit"
var m_acWavRec    = m_acWorkspace + "\\CoolEditStart.exe ";
var m_acWavPlay   = m_acPrograms + "\\WinAmp\\Winamp.exe ";
var m_acMzsExe    = m_acMzsHome + "MzfStudio.exe ";
var m_acMzsReg    = "HKCU\\Software\\UlrichKeller\\MZFStudio\\V2.4\\";
var m_acMzsRegWav = m_acMzsReg + "VerzeichnisWAVDateien"
var m_acMzsRegMzf = m_acMzsReg + "VerzeichnisMZFDateien"
var m_acFnP       = "-P";
var m_acFnR       = "-R";
var m_acFnS       = "-S";

var m_aiCpT       = [];
var m_aiCpW       = [];
var m_aiCpR       = [];

//************************************************************** implementation
//======================================================================= Mzf()
Mzf_Main();
m_oShell.Quit;
//=================================================================== eoF Mzf()

//================================================================== Mzf_Main()
// Purpose...: Main module
//=============================================================================
function Mzf_Main() {

  var fContinue = false;

  var oArgs     = {acFns:       m_acFnP + m_acFnR + m_acFnS,
                   acFn:        "",
                   acInFile:    "",
                   acInPath:    "",
                   acInName:    "",
                   acInExt:     "",
                   acInNameExt: ""
                  };

  //------------------------ prepare codepage arrays for binary file read/write
  Lib_BinFilePrep();

  //----------------------------------------------- read command line arguments
  fContinue = Mzf_GetCommandArgs(oArgs);

  //----------------------------- convert a ZEN source file and play or send it
  if ((fContinue) && (oArgs.acInExt == "ZEN") && (oArgs.acFn != m_acFnR)) {
    Mzf_Zen(oArgs);
    fContinue = false;
  }

  //--------------------------------------------------------------- record (-R)
  if ((fContinue) && (oArgs.acFn == m_acFnR)) {
    Mzf_Record(oArgs);
  }

  //------------------------------------------ convert MZF to WAV and play (-P)
  if ((fContinue) && (oArgs.acFn == m_acFnP)) {
    Mzf_Play(oArgs);
  }

  //---------------------------------- convert MZF file and send to RCVBIN (-M)
  if ((fContinue) && (oArgs.acFn == m_acFnS))  {
    Mzf_Send(oArgs);
  }
}
//============================================================== eof Mzf_Main()

//================================================================ Mzf_Record()
// Purpose...: Record a file using CoolEdit, then start MzfStudio to import
//             the recorded WAV and create the MZF file
// Input.....: i_oArgs: Command line arguments
//=============================================================================
function Mzf_Record(i_oArgs) {

  var oFile;
  var fNewMzf   = false;
  var acZen     = "";
  var acAct     = "completed";
  var acIni     = "/ini:" + m_acCoolIni;
  var acData    = " /data:" + m_acTempDir;
  var acWavE    = m_acCoolIni + "\\Empty.wav";
  var acWavT    = m_acTempDir + i_oArgs.acInName + ".WAV";
  var acInName  = i_oArgs.acInName.toUpperCase();
  var acInExt   = i_oArgs.acInExt.toUpperCase();
  var acPrj     = i_oArgs.acInPath;
  var acMzf     = acPrj + "\\" + acInName + ".MZF";
  var acRec     = acPrj + "\\" + acInName + ".REC";
  var acWav     = acPrj + "\\" + acInName + ".WAV";

  //--------------------------------------------------------------- record file
  m_oFSys.CopyFile(acWavE, acWavT, true);
  m_oShell.Run(m_acWavRec + acIni + acData + " " + acWavT, 1, true);

  oFile = m_oFSys.GetFile(acWavT);
  if (oFile.size > 1000) {

    //---------------------------------- create backup file and delete original
    if (m_oFSys.FileExists(acMzf)) {
      m_oFSys.CopyFile(acMzf, acMzf + ".bak", true);
      m_oFSys.DeleteFile(acMzf, true);
    }
    else {
      fNewMzf = true;
    }

    //------------------------------------------------------ convert WAV to MZF
    m_oShell.RegWrite(m_acMzsRegMzf, acPrj, "REG_SZ");
    m_oShell.RegWrite(m_acMzsRegWav, m_acTempDir, "REG_SZ");
    m_oShell.Run(m_acMzsExe, 1, true);

    //------------------------------ MzfStudio leaved without creating MZF file
    if ((fNewMzf == false) && (! m_oFSys.FileExists(acMzf))) {
      m_oFSys.CopyFile(acMzf + ".bak", acMzf, true);
      m_oFSys.DeleteFile(acMzf + ".bak", true);
      acAct = "cancelled";
    }

    //------------------------------------------ extract text part of .MZF file
    else if (acInExt == 'ZEN') {
      var acLine = "";
      var oStream = new ActiveXObject('ADODB.Stream');
      var oRecFile = m_oFSys.CreateTextFile(acRec, true);

      oStream.Type = 2;
      oStream.CharSet = '437';
      oStream.Open();
      oStream.LoadFromFile(acMzf);

      //------------------------------------------------- insert ZENSRC comment
      oStream.Position = 24;
      oStream.WriteText("ZENSRC");
      oStream.SaveToFile(acMzf, 2);

      oStream.Position = 128;
      while (! oStream.EOS) {
        acLine = oStream.ReadText(-2);
        oRecFile.WriteLine(acLine);
      }
      oRecFile.Close();
      oStream.Close();
      acZen = "ZEN source stored in " + i_oArgs.acInName + ".REC\n";
    }

    m_oShell.PopUp(acZen + "Recording " + acAct, 0, i_oArgs.acInName, 0);
  }

  try {
    m_oFSys.DeleteFile(acWavT);
  }
  catch (oError) {};
}
//============================================================ eoF Mzf_Record()

//================================================================== Mzf_Play()
// Purpose...: Convert a MZF file to WAV in MzfStudio and play it using WinAmp
// Input.....: i_oArgs: Command line arguments
//=============================================================================
function Mzf_Play(i_oArgs) {

  var iExeRes = 1;
  var acInFile = i_oArgs.acInPath + "\\" + i_oArgs.acInName + ".MZF";

  if (m_oFSys.FileExists(acInFile)) {
    var acWav = m_acTempDir + i_oArgs.acInName + ".WAV";
    m_oShell.RegWrite(m_acMzsRegWav, m_acTempDir, "REG_SZ");

    try {
      m_oFSys.DeleteFile(acWav);
    }
    catch (oError) {}

    m_oShell.Run(m_acMzsExe + acInFile, 1, true);

    //----------------------------------------------- play the WAV using WinAmp
    if (m_oFSys.FileExists(acWav)) {
      var acPrompt = "ML:L<CR> in Monitor  ZEN:R<CR><CR>  Basic:LOAD<CR>";
      iExeRes = m_oShell.PopUp(acPrompt, 0, i_oArgs.acInName, 1);

      if (iExeRes == 1) {
        m_oShell.Run(m_acWavPlay + acWav, 1, true);
      }

      try {
        m_oFSys.DeleteFile(acWav);
      }
      catch (oError) {};
    }
  }
  else {
    m_oShell.PopUp(i_oArgs.acInName + ".MZF not found",0,i_oArgs.acInName,0);
  }
}
//============================================================== eoF Mzf_Play()

//================================================================== Mzf_Send()
// Purpose...: Convert MZF file to binary for RCVBIN and send it
//             Supported MZF filetypes:
//             $01: Binary
//             $04: ZEN Source
//             $05: Basic Source
//             $A0: Pascal Source
// Input.....: i_oArgs: Command line arguments
// Result....: true: file was sent, false: transfer aborted in TT script
//=============================================================================
function Mzf_Send(i_oArgs) {

  var fFnResult   = false;
  var fMzfFound   = false;
  var fSend       = true;
  var fZenSrc     = false;
  var abInData    = [];
  var abOutData   = [];
  var iCnt        = 0;
  var iCsum       = 0;
  var iAddrOffs   = 0;
  var acComment   = "";
  var acInFile    = i_oArgs.acInPath + "\\" + i_oArgs.acInName + ".MZF";
  var acOutFile   = m_acTempDir + "SENDMZF.BIN";
  var acTtExe     = m_acPrograms + "\\TeraTerm\\TtpMacro.exe ";
  var acTtTtl     = acTtExe + "/V " + m_acMzsHome + "SendBin.ttl ";
  var abFileTypes = [001,004,005,160];
  var iFileType   = 0;

  //-------------------------------------- check vor valid filetype in MZF file
  if (m_oFSys.FileExists(acInFile)) {
    fMzfFound = true;
    fFnResult = false;
    Lib_BinFileRead(acInFile, abInData);
    iFileType = abInData[0];

    for (iCnt = 0; iCnt < abFileTypes.length; iCnt ++) {
      if (iFileType == abFileTypes[iCnt]) fFnResult = true;
    }
  }

  //------------------------------------------ continue if .MZF found and valid
  if (fFnResult == true) {

    //--------------------------------------------- leading NULs and start mark
    for (iCnt = 0; iCnt < 50; iCnt ++) {
      abOutData.push(0);
    }
    abOutData.push(65);
    iAddrOffs = abOutData.length;

    //-------------------- copy length, load and exec addresses to output array
    for (iCnt = 18; iCnt < 24; iCnt ++) {
      abOutData.push(abInData[iCnt]);
    }

    //---------------------------------------- copy file body and calc checksum
    for (iCnt = 128; iCnt < abInData.length; iCnt ++) {
      iCsum += abInData[iCnt];
      iCsum %= 256;
      abOutData.push(abInData[iCnt]);
    }

    //--------------------------------------- check if the file is a ZEN source
    acComment = "";
    for (iCnt = 24; iCnt < 30; iCnt ++) {
      acComment += String.fromCharCode(m_aiCpW[abInData[iCnt]]);
    }
    if (acComment == "ZENSRC") {
      fZenSrc = true;
    }

    //--------------------------------------------------- patch the output file
    iCsum += Mzf_PatchBin(abInData[0], fZenSrc, iAddrOffs, abOutData);
    iCsum %= 256;

    //------------------------------------------------------- write output file
    abOutData.push(iCsum);
    Lib_BinFileWrite(abOutData, acOutFile);

    //----------------------------------- call TeraTerm script to send the file
    if (fSend == true) {
      var acBasic = (iFileType == 5) ? " B" : "";
      m_oShell.Run(acTtTtl + acOutFile + " " + i_oArgs.acInName + acBasic,1,1);

      //------------------ the .INI file still exists if transfer was cancelled
      if (m_oFSys.FileExists(m_acTempDir + "SENDBIN.INI")) {
        m_oFSys.DeleteFile(m_acTempDir + "SENDBIN.INI");
        fFnResult = false;
      }

      try {
        m_oFSys.DeleteFile(acOutFile);
      }
      catch (oError) {}
    }
  }
  else if (fMzfFound == false) {
    acTmp = "No .MZF file in project " + i_oArgs.acInName;
    m_oShell.PopUp(acTmp, 0, i_oArgs.acInName, 0);
  }
  else if (fFnResult == false) {
    acTmp = ".MZF file is of wrong type or corrupt";
    m_oShell.PopUp(acTmp, 0, i_oArgs.acInName, 0);
  }

  return (fFnResult);
}
//============================================================== eoF Mzf_Send()

//============================================================== Mzf_PatchBin()
// Purpose...: Patch binary output array witch code to set end-of-text
// Input.....: i_iType : File type ($05= basic, $0A= pascal, $04= ZEN)
//             i_fZen  : true= it's ZEN source ($04= basic data file also)
//             i_iAdOfs: Offset in byte array where addresses start
//             i_abBin : Array containing the binary file
// Output....: i_abBin : Modified byte array
// Result....: Checksum of added bytes
//=============================================================================
function Mzf_PatchBin(i_iType, i_fZen, i_iAdOfs, i_abBin) {

  var iAddr1   = 0;
  var iAddr2   = 0;
  var iStart   = 0;
  var iSize    = 0;
  var iLoad    = 0;
  var iCode    = 0;
  var iCsum    = 0;
  var iCsStart = 0;
  var iCsEnd   = 0;

  //------------------------------------------ define patch and start addresses
  if ((i_fZen == true) && (i_iType == 4)) {      // ZEN source ($04)
    iAddr1 = 5004;                               // XEOF is at $138C
    iStart = 4608;                               // start address is $1200
  }
  else if (i_iType == 160) {                     // pascal source ($A0)
    iAddr1 = 20956;                              // end of text is at $51DC
    iAddr2 = 20958;                              // and at $51DE
    iStart = 4704;                               // pascal warm start $1260
  }

  //---------------------------------------------------------------------------
  // Basic programs require special handling because of bankswitching. If
  // programs go beyond $D000 the patch code is not accessible anymore.
  // The Basic interpreter must be patched with the patchcode which gets the
  // end address of the program, stored by RCVBIN at $10F0.
  // It then patches Basic's end-of-text address and does a Basic-warmboot.
  // There is a free area starting at $2CA9, the code is placed there
  // The version 1.2 of S-Basic has code at $2CA9, shifted up to $2CC0.
  // The basic-extender has code at $2CC0, shifted up to $2FA0
  // Patch address in MZF file is $2FA0 + $80 = $0320
  // 2FA0         LD  HL,(10F0H)   ; HL= size of basic program
  // 2FA3         LD  DE,(10F2H)   ; DE= load address
  // 2FA7         ADD HL,DE        ; HL= end address of basic program
  // 2FA8         LD  (6AB3H),HL   ; set Basic's end address
  // 2FAB         JP  00B5H        ; Basic warm-boot
  // It is not possible to check if Basic is loaded, if not, crash...
  //---------------------------------------------------------------------------
  else if (i_iType == 5) {                       // basic source ($05)
    iStart = 12192;                              // patchcode in Basic ($2FA0)
    i_abBin[i_iAdOfs+4] = (iStart & 255);        // lowbyte of start address
    i_abBin[i_iAdOfs+5] = (iStart >> 8);         // hibyte of start address
    iStart = 0;
  }

  if (iStart != 0) {                             // if known type only
    iSize  = i_abBin[i_iAdOfs];                  // size of body, low
    iSize += i_abBin[i_iAdOfs+1] * 256;          //               high
    iLoad  = i_abBin[i_iAdOfs+2];                // load address, low
    iLoad += i_abBin[i_iAdOfs+3] * 256;          //               high
    iCode  = iLoad + iSize;                      // where code goes

    //---------------------------------- write the patch program to end of body
    iCsStart = i_abBin.length;                   // startpos checksum calc
    i_abBin.push(33);                            // LD HL,iSize (21 lo hi)
    i_abBin.push(iCode & 255);                   // lowbyte of size
    i_abBin.push(iCode >> 8);                    // hibyte of size

    i_abBin.push(34);                            // LD (iAddr1),HL (22 lo hi)
    i_abBin.push(iAddr1 & 255);                  // lowbyte of iAddr1
    i_abBin.push(iAddr1 >> 8);                   // hibyte of iAddr1

    if (iAddr2 != 0) {                           // if iAddr2 used
      i_abBin.push(33);                          // LD HL,iSize (21 lo hi)
      i_abBin.push(iCode & 255);                 // lowbyte of size
      i_abBin.push(iCode >> 8);                  // hibyte of size

      i_abBin.push(34);                          // LD (iAddr2),HL (22 lo hi)
      i_abBin.push(iAddr2 & 255);                // lowbyte of iAddr2
      i_abBin.push(iAddr2 >> 8);                 // hibyte of iAddr2
    }

    if (i_fZen == true) {                        // check if ZEN is in memory
      i_abBin.push(42);                          // 2A 00 12  LD HL,($1200)
      i_abBin.push(0);
      i_abBin.push(18);
      i_abBin.push(17);                          // 11 C3 D6  LD DE,$D6C3
      i_abBin.push(195);
      i_abBin.push(214);
    }

    if (i_iType == 160) {                        // check if pascal in memory
      i_abBin.push(42);                          // 2A 00 12  LD HL,($1200)
      i_abBin.push(0);
      i_abBin.push(18);
      i_abBin.push(17);                          // 11 18 02  LD DE,$0218
      i_abBin.push(24);
      i_abBin.push(02);
    }

    i_abBin.push(175);                           // AF        XOR A
    i_abBin.push(237);                           // ED 52     SBC HL,DE
    i_abBin.push(82);
    i_abBin.push(194);                           // C2 AD 00  JP NZ,$00AD
    i_abBin.push(173);
    i_abBin.push(0);

    i_abBin.push(195);                           // JP iStart (C3 lo hi)
    i_abBin.push(iStart & 255);                  // lowbyte of start address
    i_abBin.push(iStart >> 8);                   // hibyte of start address
    iCsEnd = i_abBin.length;                     // end pos checksum calc

    //---------------------------------------------------- modify start address
    i_abBin[i_iAdOfs+4] = iCode & 255;           // lowbyte of start address
    i_abBin[i_iAdOfs+5] = iCode >> 8;            // hibyte of start address

    //-------------------------------------------------------- modify file size
    iSize += (iCsEnd-iCsStart);                  // the code size
    i_abBin[i_iAdOfs] = iSize & 255;             // lowbyte of new size
    i_abBin[i_iAdOfs+1] = iSize >> 8;            // hibyte of new size

    //------------------------------------------- calc checkum of inserted code
    for (var iCnt = iCsStart; iCnt < iCsEnd; iCnt ++) {
      iCsum += i_abBin[iCnt];
      iCsum %= 256;
    }
  }
  return (iCsum);
}
//========================================================== eoF Mzf_PatchBin()

//=================================================================== Mzf_Zen()
// Purpose...: Play or send a ZEN source file
// Input.....: i_oArgs: Command line arguments
//=============================================================================
function Mzf_Zen(i_oArgs) {

  var oInFile;
  var oZenFile;
  var oStream   = new ActiveXObject('ADODB.Stream');
  var abHdr     = new Array(128);
  var iNameLen  = i_oArgs.acInName.length;
  var acOut     = "";
  var cTmp      = '';
  var acTmp     = "";
  var acComment = "";
  var iCnt      = 0;
  var iNdx      = 0;
  var iPos      = 0;
  var iSize     = 0;

  //-------------------------------------------------- create tape header array
  for (iNdx = 0; iNdx < 128; iNdx++) {
    abHdr[iNdx] = 0;
  }

  abHdr[0] = 4;                                  // file type, fixed to 4
  iNdx = 1;                                      // filename begins here

  for (iCnt = 0; iCnt < iNameLen; iCnt ++) {     // copy filename to array
    abHdr[iNdx] = i_oArgs.acInName.charCodeAt(iCnt);
    iNdx ++;
  }
  abHdr[iNdx] = 13;                              // filename ends with CR

  oInFile = m_oFSys.GetFile(i_oArgs.acInFile);   // get infile properties
  iSize = oInFile.size;
  abHdr[18] = iSize & 255;                       // lowbyte of filesize
  abHdr[19] = iSize >> 8;                        // hibyte of filesize
  abHdr[20] = 0;                                 // lowbyte of load address
  abHdr[21] = 48;                                // hibyte, fixed to $3000
  abHdr[22] = 0;                                 // lowbyte of start address
  abHdr[23] = 18;                                // hibyte, fixed to $1200

  //--------------------------------------------- write header to stream object
  for (iNdx = 0; iNdx < abHdr.length; iNdx++) {
    acOut += String.fromCharCode(m_aiCpW[abHdr[iNdx]]);
  }

  oStream.Type = 2;
  oStream.CharSet = '437';
  oStream.Open();
  oStream.WriteText(acOut);

  //---------------------------------------------- append source file to stream
  oZenFile = m_oFSys.OpenTextFile(i_oArgs.acInFile, 1);
  while (! oZenFile.AtEndOfStream) {
    acOut = oZenFile.ReadLine();
    acOut += String.fromCharCode(m_aiCpW[13]);
    acOut = acOut.toUpperCase();

    //------------------------------------------------ uncomment LOAD after ORG
    iPos = acOut.indexOf("LOAD ");
    if (iPos != -1) {
      if (acOut.indexOf(';') == 0) {
        acOut = ' ' + acOut.substr(1);
      }
    }

    //-------------------------------------------- replace 'PHASE ' with 'ORG '
    iPos = acOut.indexOf("PHASE ");
    if (iPos != -1) {
      acTmp  = acOut.substr(0,iPos);
      acTmp += 'ORG ';
      acTmp += acOut.substr(iPos + 5);
      acOut = acTmp;
    }

    //------------------------------------------------ replace 'AF'' with 'AF '
    iPos = acOut.indexOf("AF'");
    if (iPos != -1) {
      acTmp  = acOut.substr(0,iPos);
      acTmp += 'AF ';
      acTmp += acOut.substr(iPos + 3);
      acOut = acTmp;
    }

    //------------------------------------------------ skip lines with ';;' tag
    iPos = acOut.indexOf(";;");
    if (iPos == -1) {
      oStream.WriteText(acOut);
    }
  }
  oZenFile.Close();

  //----------------------------------------------------- insert ZENSRC comment
  oStream.Position = 24;
  oStream.WriteText("ZENSRC");

  //-------------------------------------- correct filesize, might have changed
  iSize = oStream.Size - 128;
  oStream.Position = 18;
  oStream.WriteText(String.fromCharCode(m_aiCpW[iSize % 256]));
  oStream.Position = 19;
  oStream.WriteText(String.fromCharCode(m_aiCpW[iSize >> 8]));

  //-------------------------------------------------------- save stream to MZF
  acOut = i_oArgs.acInPath + "\\" + i_oArgs.acInName + ".MZF";
  oStream.SaveToFile(acOut, 2);
  oStream.Close();

  //------------------------------------------------------- send file to RCVBIN
  if (i_oArgs.acFn == m_acFnS) {
    Mzf_Send(i_oArgs, true);
  }

  //------------------------------------------------------------------ play WAV
  else {
    i_oArgs.acInFile = acOut;
    Mzf_Play(i_oArgs);
  }
}
//=============================================================== eoF Mzf_Zen()

//======================================================== Mzf_GetCommandArgs()
// Purpose...: Command line parser
// Input.....: i_oArgs: Object with keys and return values
// Result....: true= no errors, false= some error
//=============================================================================
function Mzf_GetCommandArgs(i_oArgs) {

  var fFnResult = false;

  if ((m_oArgs.length) == 2) {
    fFnResult = true;
    i_oArgs.acFn = m_oArgs(0);
    i_oArgs.acFn.toUpperCase();

    if (i_oArgs.acFns.indexOf(i_oArgs.acFn) == -1) {
      WScript.Echo("Invalid Command (not of " + i_oArgs.acFns + ")");
      fFnResult = false;
    }

    if (fFnResult == true) {
      i_oArgs.acInFile = m_oArgs(1);
      i_oArgs.acInFile.toUpperCase();
      i_oArgs.acInPath = m_oFSys.GetParentFolderName(i_oArgs.acInFile);
      i_oArgs.acInName = m_oFSys.GetBaseName(i_oArgs.acInFile);
      i_oArgs.acInExt = m_oFSys.GetExtensionName(i_oArgs.acInFile);
      i_oArgs.acInNameExt = i_oArgs.acInName + "." + i_oArgs.acInExt;

      if (! m_oFSys.FileExists(i_oArgs.acInFile)) {
        m_oShell.PopUp(i_oArgs.acInName + ".MZF not found", 0, "Error", 0);
        fFnResult = false;
      }
    }
  }
  else {
    WScript.Echo("Usage: Mzf.js <command> <InFile>");
  }

  return (fFnResult);
}
//==================================================== eoF Mzf_GetCommandArgs()

//++++++++++++++++++++++++++++++++++++++++++++++++++++++ binary file read/write
// code found at stackoverflow.com/questions/42225862/
//               how-to-write-a-sequence-of-bytes-to-binary-file-using-jscript/
//               42327601#42327601
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//=========================================================== Lib_BinFilePrep()
// Purpose.....: Prepare codepage tables for binary file read/write
// Input.......: m_aiCpT, m_aiCpW, m_aiCpR must be defined as global arrays
// Output......: m_aiCpT, m_aiCpW, m_aiCpR contain codepage tables
//=============================================================================
function Lib_BinFilePrep() {

  var iCnt = 0;
  var iLen = 0;
  var iCtb = 0;

  m_aiCpT = [ 199, 252, 233, 226, 228, 224, 229, 231, 234, 235, 232, 239, 238,
              236, 196, 197, 201, 230, 198, 244, 246, 242, 251, 249, 255, 214,
              220, 162, 163, 165,8359, 402, 225, 237, 243, 250, 241, 209, 170,
              186, 191,8976, 172, 189, 188, 161, 171, 187,9617,9618,9619,9474,
             9508,9569,9570,9558,9557,9571,9553,9559,9565,9564,9563,9488,9492,
             9524,9516,9500,9472,9532,9566,9567,9562,9556,9577,9574,9568,9552,
             9580,9575,9576,9572,9573,9561,9560,9554,9555,9579,9578,9496,9484,
             9608,9604,9612,9616,9600, 945, 223, 915, 960, 931, 963, 181, 964,
              934, 920, 937, 948,8734, 966, 949,8745,8801, 177,8805,8804,8992,
             8993, 247,8776, 176,8729, 183,8730,8319, 178,9632, 160
            ];

  //------------------- create forward lookup to write & reverse lookup to read
  for (iCnt = 0, iLen = 256; iCnt < iLen; iCnt++) {
    iCtb = (iCnt < 128) ? iCnt : m_aiCpT[iCnt-128];
    m_aiCpW[iCnt] = iCtb;
    m_aiCpR[iCtb] = iCnt;
  }
}
//======================================================== eoF LibBinFilePrep()

//=========================================================== Lib_BinFileRead()
// Purpose.....: Read a file into a array of bytes
// Input.......: i_acFileName: File with path to read from
//               i_abArray: Array of bytes to read
// Output......: i_abArray: Holds file data
//=============================================================================
function Lib_BinFileRead(i_acFileName, i_abArray) {

  var inStream = new ActiveXObject("ADODB.Stream");
  inStream.Type = 2;
  inStream.CharSet = '437';
  inStream.Open();
  inStream.LoadFromFile(i_acFileName);
  var inString = inStream.ReadText();
  inStream.Close();

  for (var i = 0, l = inString.length; i < l; i++) {
    i_abArray.push(m_aiCpR[inString.charCodeAt(i)]);
  }
}
//======================================================= eoF Lib_BinFileRead()

//========================================================== Lib_BinFileWrite()
// Purpose.....: Write a array of bytes to a file
// Input.......: i_abArray: Array of bytes to write
//               i_acFileName: File with path to write to
//=============================================================================
function Lib_BinFileWrite(i_abArray, i_acFileName) {

  var outStream = new ActiveXObject('ADODB.Stream');
  var outString = '';

  for (var i = 0, l = i_abArray.length; i < l; i++) {
    outString += String.fromCharCode(m_aiCpW[i_abArray[i]]);
  }

  outStream.Type = 2;
  outStream.CharSet = '437';
  outStream.Open();
  outStream.WriteText(outString);
  outStream.SaveToFile(i_acFileName, 2);
  outStream.Close();
}
//====================================================== eoF Lib_BinFileWrite()
//++++++++++++++++++++++++++++++++++++++++++++++++++ end binary file read/write

//################################################################## EOF Mzf.js